#!/usr/bin/env python
# coding=utf-8

"""
SPLITDJVU 0.01

Tool for converting pdf or djvu books with 2-in-1 pages into djvu with solitary pages.

Usage:

splitdjvu.py infile [palettefile] [noclean]

(if this program file lies in system PATH (e. g. /usr/bin))

or

./splitdjvu.py infile [palettefile] [noclean]

(if this program file is in the same directory)

or

python splitdjvu.py infile [palettefile] [noclean]

(if your system does not want to execute it in another way).

Examples:

splitdjvu.py "Ray.(1995).An Evolutionary Approach to Synthetic Biology.Zen and the Art of Creating Life.pdf"

splitdjvu.py "Солбриг, О. и Д. - Популяционная биология и эволюция (М., 1982).djvu" grayscale.ppm

splitdjvu.py "Солбриг, О. и Д. - Популяционная биология и эволюция (М., 1982).djvu" noclean

(in the last case cleaning of intermediate files won't be performed).

Palette file should be .ppm image file from which all colors will be taken.
Default palette is bw.ppm which should be shipped with this program and lie in current directory.

Required libraries: djvulibre imagemagick (maybe also libtiff and something else, google for unfound executables)

Not works in windows yet (but maybe will be in future).

See http://www.onyx-boox.ru/board/viewtopic.php?sid=33f1c235c4f5f97c8a67b7966c0c1d29&p=1801250#1801250 for support.
"""

import os

def tifpaldjvu(tiffile, palfile = None, colors = 32, tifext = "tif", ppmext = "ppm", djvext = "djv", clean = True):
    """Converts tiff file to one-paged djvu according to palette (good for few-colored books)
    palette is ppm image file
    example usage:
    
    files = sorted(filter(lambda a: ".tif" in a, os.listdir(".")))
    
    for f in files:
        tifpaldjvu(f, "../palette.ppm")
    
    djvm -c "book.djvu" *.djv # in command line
    
    Parameters:
    
    tiffile - input tif file name
    palfile - palette ppm image file name
    colors - maximum number of colors on page. If palette is used, number of colors will be minimum of this parameter and colors in palette. Otherwise colors will be recognized automatically (and sometimes quite poorly).
    tifext etc. - extensions of file names.
    clean - whether to clean intermediate ppm files (recommended to be true cos they are large and contain no additional information).
    """
    ppmfile = tiffile.replace(tifext, ppmext)
    ppm2file = "_" + ppmfile
    djvfile = tiffile.replace(tifext, djvext)
    os.system("convert -format ppm " + tiffile + " " + ppmfile)
    if palfile:
        os.system("ppmquant -mapfile " + palfile + " " + ppmfile + " > " + ppm2file)
    else:
        os.system("ppmquant " + str(colors) + " " + ppmfile + " > " + ppm2file)
    os.system("cpaldjvu -colors  " + str(colors) + " " + ppm2file + " " + djvfile)
    if clean:
        os.system("rm " + ppmfile + " " + ppm2file)

def tifsplitdjvu(tiffile, palfile = None, colors = 16, cleansrc = True, cleanppm = True):
    """Splits tiff file to left and right halves.
    
    Parameters are like tifpaldjvu.
    cleanppm is for cleaning ppm files, cleansrc is for cleaning source tiff file.
    """
    fl = tiffile.replace(".tif", "_L.tif")
    fr = tiffile.replace(".tif", "_R.tif")
    os.system("convert -crop 50%x100% -gravity West " + tiffile + " " + fl)
    os.system("convert -crop 50%x100% -gravity East " + tiffile + " " + fr)
    tifpaldjvu(fl, palfile, colors, clean = cleanppm)
    tifpaldjvu(fr, palfile, colors, clean = cleanppm)
    if cleansrc:
        os.system("rm " + tiffile)

def djvusplit(djvfile, palfile = None, outfile = None, colors = 16, tmpdir = "./djvusplit_temp", clean = True, cleanppm = True):
    """Converts djvu file with 2-in-1 pages into djvu with solitary pages.
    
    Parameters are like tifpaldjvu.
    cleanppm is for cleaning ppm files.
    clean is for cleaning temporary directory with intermediate tiff and djvu files.
    
    Temporary directory (djvusplit_temp) should not exist before executing.
    """
    wholetif = djvfile + ".tif"
    os.mkdir(tmpdir)
    #print("ddjvu -format=tiff '" + djvfile + "' '" + tmpdir + "/" + wholetif + "'")
    os.system("ddjvu -format=tiff '" + djvfile + "' '" + tmpdir + "/" + wholetif + "'")
    os.chdir(tmpdir)
    os.system("tiffsplit '" + wholetif + "'")
    os.system("rm '" + wholetif + "'")
    
    files = sorted(filter(lambda a: ".tif" in a, os.listdir(".")))
    #files.remove(wholetif)
    if palfile:
        palfile = "../" + palfile
    for f in files:
        tifsplitdjvu(f, palfile, colors, cleanppm = cleanppm)
    
    if outfile == None:
        outfile = djvfile.replace(".djv", "_split.djv")
    os.system("djvm -c '../" + outfile + "' *.djv")
    os.chdir("..")
    if clean:
        os.system("rm -rf " + tmpdir)

def pdfsplit(pdffile, dpi = 600, palfile = None, outfile = None, colors = 16, tmpdir = "./djvusplit_temp", clean = True, cleanppm = True):
    """Converts pdf file into djvu and passes it into djvusplit.
    Default dpi is 600, change it if it is needed.
    Other parameters are like in djvusplit."""
    djvfile = pdffile.replace(".pdf", ".djvu")
    os.system("pdf2djvu -d " + str(dpi) + " -o '" + djvfile + "' '" + pdffile + "'")
    djvusplit(djvfile, palfile, outfile, colors, tmpdir, clean, cleanppm)

# пример команды перевода пдф в дежавю
# pdf2djvu -d 600 -o "John Maynard Smith _Evolution and the Theory of Games_[OCR].djvu" "John Maynard Smith _Evolution and the Theory of Games_ [OCR].pdf"

# и обратно!
# ddjvu -format=pdf "John Maynard Smith _Evolution and the Theory of Games_[OCR]_split.djvu" "John Maynard Smith _Evolution and the Theory of Games_[OCR]_split.pdf"

import sys


if __name__ == "__main__":
    if len(sys.argv)<2:
        print __doc__
    else:
        #print sys.argv[1]
        infile = sys.argv[1]
        palfile = "bw.ppm"
        
        if (len(sys.argv)>2) and (sys.argv[2] != "noclean"):
            palfile = sys.argv[2]
            
        #if (len(sys.argv)>3) and (sys.argv[3] == "noclean"):
        if "noclean" in sys.argv:
            clean = False
        else:
            clean = True
        
        if ".pdf" in infile:
            print "\nSplitting " + infile + " PDF file using " + palfile + " palette file"
            pdfsplit(infile, palfile = palfile, clean = clean)
        else:
            print "\nSplitting " + infile + " DJVU file using " + palfile + " palette file"
            djvusplit(infile, palfile = palfile, clean = clean)
        
        print "All should be done!"

